'' =================================================================================================
''
''   File....... sk6812_multi_strip_demo.spin
''   Purpose.... demonstration program to run multiple sk6812rgbw and/or sk6812/ws2812b LED strips 

''               Based on code by Jon McPhalen

''   Edited by.. Darren Payson
''               Copyright (c) 2016-2018 Darren Payson                  
''               -- see below for terms of use
''   E-mail..... darren@paysonplace.net
''   Started.... 5_15_2016
''   Updated.... 
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
  

con { io pins }

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30
  
  SDA  = 29                                                     ' eeprom / i2c
  SCL  = 28
  
obj

' main                                                          ' * master Spin cog
  time  : "jm_time"                                             '   timing and delays
  io    : "jm_io_basic"                                         '   essential io
  strip : "sk6812_multi_strip"                             ' * SK2812RGBW LED driver
                                                                 
' * uses cog when loaded

con

'' Settings used to set strip direction
  frwd =  4
  rvrs = -4
  tggl = -1
  dflt =  0
   
con

   STRIP_QTY = 8  ' how many strips to process default is 8, if changed please change in object sk6812_multi_strip
 
dat               
''Enter strip transmit pins, number of LEDs per strip, if the strip is RGBW, and default direction of strip below.
        
''                         tx pin   # of LEDs    White LEDs*?  Default direction
  Strip_Params   long        0    ,      8    ,     true    ,      frwd   '   strip 0
                 long        1    ,     12    ,     true    ,      frwd   '   strip 1
                 long        2    ,     16    ,     true    ,      frwd   '   strip 2
                 long        3    ,     24    ,     true    ,      frwd   '   strip 3
                 long        4    ,     60    ,     true    ,      frwd   '   strip 4
                 long        5    ,     30    ,     true    ,      frwd   '   strip 5
                 long        6    ,     60    ,     true    ,      frwd   '   strip 6
                 long        7    ,    144    ,     true    ,      frwd   '   strip 7
                                              '*set true for sk_6812rgbw
                                              '*set false for sk_6812 and ws_2812/b
                                               
  Chakras       long    strip#RED, strip#ORANGE, strip#YELLOW 
                long    strip#GREEN, strip#BLUE, strip#INDIGO

                                      
pub main | pos, st, c_rgbw, idx
        
  setup

       color_wipe($20_00_00_00, 15)           'fill all strips with red
       color_wipe($00_20_00_00, 15)           'fill all strips with green
       color_wipe($00_00_20_00, 15)           'fill all strips with blue  
       color_wipe($00_00_00_20, 15)           'fill all rgbw strips with white
       
' change direction of every other strip
      
        repeat st from 0 to STRIP_QTY          
         strip.dir(st,tggl)
         st += 2

' run same color_wipes as above but now every other strips direction is reversed
             
       color_wipe($20_00_00_00, 15)
       strip.dir_all(tggl)                    'toggle every strips direction
       color_wipe($00_20_00_00, 15)
       strip.dir_all(tggl)        
       color_wipe($00_00_20_00, 15)
       strip.dir_all(tggl) 
       color_wipe($00_00_00_20, 15)
       
         repeat 5
          color_chase (@Chakras , 6, 50)                
       strip.dir_all(tggl)                   'toggle every strips direction 
         repeat 5
          color_chase (@Chakras , 6, 50)                
       strip.dir_all(tggl)                   'toggle every strips direction 
         repeat 5
          color_chase (@Chakras , 6, 50)         
       strip.dir_all(rvrs)                   'set every strips direction to reverse 
         repeat 5
          color_chase (@Chakras , 6, 50)         
       strip.dir_all(frwd)                   'set every strips direction to forward
        repeat 5
          color_chase (@Chakras , 6, 50)         
         
     st := 0
     idx:= 0

'' starting with strip 0 fades each pixel in strip to color from @Chakras
          
     repeat
       repeat                                                     'begins repeat loop 
        time.pause(10)                                             
       until strip.fade_strip (st,long[@Chakras][idx], 1,2) == 0  'repeats loop until leds_chngd in fade_strip method = 0    
        if (++st == STRIP_QTY)
         st := 0                                                  'starts at strip 0 again
        if (++idx == 6)
         idx := 0                                                 'starts at @Chakras index 0 again


pub color_chase(p_colors, len, ms) | base, idx, ch

'' Performs color chase 

  repeat base from 0 to len-1                                   ' do all colors in table
    idx := base                                                 ' start at base
    repeat ch from 0 to strip.num_pixels-1                      ' loop through connected leds
      strip.set(ch, long[p_colors][idx])                        ' update channel color 
      if (++idx == len)                                         ' past end of list?
        idx := 0                                                ' yes, reset
   
    time.pause(ms)                                              ' set movement speed


pub setup                                                                                                         
          
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  io.start(0, 0)                                                ' clear all pins (master cog)

  strip.start (@Strip_Params)                                 ' start led driver (point to Strip_Params index) 

  strip.off

       
con

  ' Routines ported from C code by Phil Burgess (www.paintyourdragon.com)


pub color_wipe(rgbw, ms) | ch

'' Sequentially fills strip with color rgbw
'' -- ms is delay between pixels, in milliseconds

  repeat ch from 0 to strip.num_pixels-1 
    strip.set(ch, rgbw)
    time.pause(ms)


pub rainbow(ms) | pos, ch

  repeat pos from 0 to 255
    repeat ch from 0 to strip.num_pixels-1
      strip.set(ch, strip.wheel((pos + ch) & $FF))
    time.pause(ms)
    

pub rainbow_cycle(ms) | pos, ch 

  repeat pos from 0 to (255 * 5)
    repeat ch from 0 to strip.num_pixels-1
      strip.set(ch, strip.wheel(((ch * 256 / strip.num_pixels) + pos) & $FF))
    time.pause(ms)
    

dat { license }

{{

  Copyright (c) 2016-2018 Darren Payson 

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}